#!/usr/bin/env node

const pjson = require('./package.json');
const execa = require('execa');
const Listr = require('listr');
const yargs = require('yargs');
const copyDir = require('copy-dir');
const fs = require('fs');
const cwd = process.cwd();

const tasks = new Listr([
  {
    title: 'initialize',
    task: () => {
      copyDir.sync(`${__dirname}/boilerplate`, cwd);
      return true;
    },
  },
  {
    title: 'copy .gitignore',
    task: () => {
      fs
        .createReadStream(`${__dirname}/templates/_gitignore`)
        .pipe(fs.createWriteStream(`${cwd}/.gitignore`));
    },
  },
  {
    title: 'Install dependencies',
    task: (ctx, task) =>
      execa('yarn', ['add', `applibook@${pjson.version}`]).catch(() => {
        ctx.yarn === false;
        task.skip('Yarn not available, install it via `npm install -g yarn`');
      }),
  },
]);

yargs
  .command('create', 'create a new gitbook', yargs => {
    console.log('create');
    tasks.run().catch(err => {
      console.error(err);
    });
  })
  .command('*', 'get cli version', yargs => {
    console.log(`applibok version: ${pjson.version}`);
    console.log('run `applibook --help` to see avaibale commands');
  })
  .help().argv;
